:root {
    --text-color-override: #428cdb;
    --title-color: #428cdb;
    /* --text-color-override: #38a7d2; */
}

article {
    line-height: 1.6;
}

ul {
    padding-left: 2rem;
}

p {
    margin: 0 0 18px;
}

h1,
h2,
h3,
h4,
h5 {
    padding-top: 8px;
    padding-bottom: 8px;
    color: var(--title-color);
}

.sidebar-brand-text {
    text-align: center;
}

.sidebar-brand-text[data-theme="light"] {
    color: #444 !important;
}

.sidebar-brand-text[data-theme="dark"] {
    color: #FFF !important;
}

.sidebar-tree .toctree-l1 > .reference,
.sidebar-tree .toctree-l2 > .reference,
.sidebar-tree .toctree-l3 > .reference {
    color: var(--text-color-override);
}

.sidebar-logo {
    width: 160px;
}

.admonition {
    padding: 0 0.9rem 0.9rem;
}

p.admonition-title {
    margin: 0 -0.9rem 0.8rem -0.9rem
}

.clear-image {
    padding-top: 35px;
    padding-bottom: 35px;
}

.clear-title {
    padding-top: 35px;
    padding-bottom: 50px;
}

.light-bg {
    background-color: #FFF;
}